<?php

namespace common\logic\User\CreditHistory\Service;

use common\helpers\MeanPayment;
use common\logic\AbstractService;
use common\logic\Order\Order\Model\Order;
use common\logic\Order\Order\Service\OrderSolver;
use common\logic\User\CreditHistory\Model\CreditHistory;
use common\logic\User\User\Model\User;
use common\logic\UtilsInterface;

class CreditUtils extends AbstractService implements UtilsInterface
{
    protected CreditHistoryBuilder $creditHistoryBuilder;
    protected OrderSolver $orderSolver;

    public function loadDependencies(): void
    {
        $this->creditHistoryBuilder = $this->loadService(CreditHistoryBuilder::class);
        $this->orderSolver = $this->loadService(OrderSolver::class);
    }

    public function creditCredit(float $amount, User $user, User $userAction, string $meanPayment): void
    {
        $this->creditHistoryBuilder->createCreditHistory(
            CreditHistory::TYPE_CREDIT,
            $amount,
            $this->getProducerContext(),
            $user,
            $userAction,
            $meanPayment
        );
    }

    public function debitCredit(float $amount, User $user, User $userAction, string $meanPayment): void
    {
        $this->creditHistoryBuilder->createCreditHistory(
            CreditHistory::TYPE_DEBIT,
            $amount,
            $this->getProducerContext(),
            $user,
            $userAction,
            $meanPayment
        );
    }

    public function payOrder(Order $order, User $userAction): void
    {
        $this->creditHistoryBuilder->createCreditHistory(
            CreditHistory::TYPE_PAYMENT,
            $this->orderSolver->getOrderAmountWithTax($order),
            $this->getProducerContext(),
            $order->user,
            $userAction,
            MeanPayment::CREDIT,
            $order
        );
    }

    public function refundOrder(Order $order, User $userAction): void
    {
        $this->creditHistoryBuilder->createCreditHistory(
            CreditHistory::TYPE_REFUND,
            $this->orderSolver->getOrderAmountWithTax($order, Order::AMOUNT_PAID),
            $this->getProducerContext(),
            $order->user,
            $userAction,
            MeanPayment::CREDIT,
            $order
        );
    }
}